//+------------------------------------------------------------------+
//|                                               ScalpingSystem.mq4 |
//|                                       Copyright 2019, Dp_Candae. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, Dp_Candae."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include <stdlib.mqh>

//external Variables
extern int StopLoss         = 50;
extern int TakeProfit       = 170;
extern double FixLotSize    = 0.10;

extern bool DynamicLotSize  = True;
extern double EquityPercent = 2;

//external variables for filterchart
extern ENUM_TIMEFRAMES TradeTimeFrame = PERIOD_M15;
extern int TradeShift                 = 0;

//external variables for filterchart
extern ENUM_TIMEFRAMES FilterTimeFrame = 1440;
extern int FilterShift                 = 49;

//Globle variables
int BuyTicket;
int SellTicket;

int MagicNumber = 01;

double UsePoint;

int ErrorCode;
  
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
      UsePoint = PipPoint(Symbol());
  }

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
  
  /*
   If DynamicLotSize is set to true, we will calculate the lot size based on the
   stop loss, and assign that value to the LotSize variable. If DynamicLotSize 
   is false, we simply assign the value of FixedLotSize to LotSize. The LotSize 
   variable will be passed to the OrderSend() function as the lot size for the order.
  */
  
  //Lot Size Calculation
  if(DynamicLotSize == True)
   {
      double RiskAmount = AccountEquity() * (EquityPercent / 100);
      double TickValue = MarketInfo(Symbol(),MODE_TICKVALUE);
      if(Digits == 3 || Digits == 5) TickValue *= 10;
      double CalcLots = (RiskAmount / StopLoss) / TickValue;
      double LotSize = CalcLots;
   }
  else LotSize = FixLotSize;
  
  //Lot size verification
  if(LotSize < MarketInfo(Symbol(),MODE_MINLOT))
  {
   LotSize = MarketInfo(Symbol(),MODE_MINLOT);
  }
  
 else if(LotSize > MarketInfo(Symbol(),MODE_MAXLOT))
 {
   LotSize = MarketInfo(Symbol(),MODE_MAXLOT);
 }
 
 if(MarketInfo(Symbol(),MODE_LOTSTEP) == 0.1)
 {
   LotSize = NormalizeDouble(LotSize,1);
 }
else LotSize = NormalizeDouble(LotSize,2);
  
   //We create a string variable for the signal
   string signal="";
   
   //Bar on the current TimeFrame
   double TradeCO  = iOpen(_Symbol, TradeTimeFrame, TradeShift);
   double TradeCC = iClose(_Symbol, TradeTimeFrame, TradeShift);
   double TradeCH  = iHigh(_Symbol, TradeTimeFrame, TradeShift);
   double TradeCL   = iLow(_Symbol, TradeTimeFrame, TradeShift);
   
   //Bar on the Higher TimeFrame
   double FilterCO  = iOpen(_Symbol, FilterTimeFrame, FilterShift);
   double FilterCC = iClose(_Symbol, FilterTimeFrame, FilterShift);
   double FilterCH  = iHigh(_Symbol, FilterTimeFrame, FilterShift);
   double FilterCL   = iLow(_Symbol, FilterTimeFrame, FilterShift);
  
 
   //converting open and close candle of tradetimeframe
  double TBulls = (TradeCO < TradeCC && TradeCH > TradeCL); 
  double TBears = (TradeCO > TradeCC && TradeCH > TradeCL);
  
  //that of Filtertimeframe
  double FBulls = (FilterCO < FilterCC && FilterCH > FilterCL);
  double FBears = (FilterCO > FilterCC && FilterCH > FilterCL);
  
  
  
  //If filter Chart bull and Main Chart Bull 
  if(TBulls && FBulls)
  {
         signal = "Buy";
       
  }
 
  if(TBears && FBears)
  {
      signal = "Sell";
  }
  
  //Place Buy Order
   if(signal == "Buy" && BuyTicket == 0)
   {
      OrderSelect(SellTicket, SELECT_BY_TICKET);
      
      //Close Order
      if(OrderCloseTime() == 0 && SellTicket > 0)
         {
            double CloseLots = OrderLots();
            
            while(IsTradeContextBusy()) Sleep(10);
            
            RefreshRates();
            double ClosePrice = Ask;
            
            bool Closed = OrderClose(SellTicket,CloseLots,ClosePrice,0,Red);
            
            
          //Error Handling
            if(Closed == False)
              {
               ErrorCode = GetLastError();
               string ErrDesc = ErrorDescription(ErrorCode);
               
               string ErrAlert = StringConcatenate("Close Sell Order - Error ",
                  ErrorCode,": ",ErrDesc);
               Alert(ErrAlert);
               
               string ErrLog = StringConcatenate("Ask: ",Ask," Lots: ",LotSize,
               " Ticket: ",SellTicket);
               Print(ErrLog);
              }
         }
         
     //Open Buy Order
     while(IsTradeContextBusy()) Sleep(10);
     RefreshRates();
     
     //Open the buy market order using OrderSend
        BuyTicket = OrderSend(Symbol(),OP_BUY,LotSize,Ask,0,0
        ,0,"Buy Order",MagicNumber,0,clrGreen); 
        
    //Error Handling
    if(BuyTicket == -1)
    {
      ErrorCode = GetLastError();
      ErrDesc = ErrorDescription(ErrorCode);
      ErrAlert = StringConcatenate("Open Buy Order - Error ",
         ErrorCode,": ",ErrDesc);
      Alert(ErrAlert);
      
      ErrLog = StringConcatenate("Ask: ",Ask," Lots: ", LotSize);
      Print(ErrLog);
    }
    
    //Order Modification Block
    else
    {
      //We Select the Order that was just placed using OrderSelect()
      OrderSelect(BuyTicket,SELECT_BY_TICKET);
      double OpenPrice = OrderOpenPrice();
      
      //Retrieve the stop level for a currency Symbol.and converting 
      //to a fractional value using Point.
      double StopLevel = MarketInfo(Symbol(),MODE_STOPLEVEL)*Point;
      
      RefreshRates();
//=====================================================================================     
      /*
      This code will calculate the minimum allowed price for a buy take profit, 
      sell stop loss, buy stop order, or sell limit order.
   */ 
//If we are placing a buy take profit with this order, then it must be
//above this price. We'll call this the UpperStopLevel, since it is above the price.   
      double UpperStopLevel = Ask + StopLevel;
/*
  This code will calculate the maximum allowed price for a sell take profit, buy stop 
  loss, sell stop order or sell limit order. Note that we are simply using the Bid 
  instead of the Ask, and subtracting instead of adding.
*/
      double LowerStopLevel = Bid - StopLevel;
//======================================================================================
     
//check the stop loss and take profit for a buy order to make sure the prices are valid.      
      double MinStop = 5 * UsePoint;
 
 //Calculate Stop Loss And TakeProfit
 if(StopLoss > 0) double BuyStopLoss = OpenPrice - (StopLoss * UsePoint);
 if(TakeProfit > 0) double BuyTakeProfit = OpenPrice + (TakeProfit * UsePoint);
 
      
      if(BuyStopLoss > 0 && BuyStopLoss > LowerStopLevel)
        {
         BuyStopLoss = LowerStopLevel - MinStop;
        }
        
      if(BuyTakeProfit > 0 && BuyTakeProfit < UpperStopLevel)
         {
            BuyTakeProfit = UpperStopLevel + MinStop;
         }
      
      //Modify Order
      if(IsTradeContextBusy()) Sleep(10);
      
      if(BuyStopLoss > 0 || BuyTakeProfit > 0)
       {
         bool TicketMod = OrderModify(BuyTicket,OpenPrice,BuyStopLoss,
            BuyTakeProfit,0);
            
   //error handling bock deals with errors from the order modification.
      //Error handling
         if(TicketMod == false)
         {
            ErrorCode = GetLastError();
            ErrDesc = ErrorDescription(ErrorCode);
            ErrAlert = StringConcatenate("Modify Buy Order - Error ",
               ErrorCode,": ",ErrDesc);
           Alert(ErrAlert);
           
           ErrLog = StringConcatenate("Ask: ",Ask," Bid: ",Bid," Ticket: ",
            BuyTicket," Stop: ",BuyStopLoss," Profit: ",BuyTakeProfit);
           Print(ErrLog);
         }
       }
    }
    SellTicket = 0;
  }
  
//===================================================================================================
  
  //Place Sell Order
  if(signal == "Sell" && SellTicket == 0)
   {
      OrderSelect(BuyTicket, SELECT_BY_TICKET);
      
      
      if(OrderCloseTime() == 0 && BuyTicket > 0)
      {
         CloseLots = OrderLots();
         
         while(IsTradeContextBusy()) Sleep(10);
         
         RefreshRates();
         ClosePrice = Bid;
         
         Closed = OrderClose(BuyTicket,CloseLots,ClosePrice,0,Red);
         
         //Error Handling
         if(Closed == false)
          {
           
            ErrorCode = GetLastError();
            ErrDesc = ErrorDescription(ErrorCode);
            
            ErrAlert = StringConcatenate("Close Buy Order - Error ",
               ErrorCode,": ",ErrDesc);
            Alert(ErrAlert);
            
            ErrLog = StringConcatenate("Bid: ",Bid," Lots: ",LotSize,
               " Ticket: ",SellTicket);
            Print(ErrLog);
          }
      }
      
      //Open Sell Order
      while(IsTradeContextBusy()) Sleep(10);
      RefreshRates();
      //OpenPrice = OrderOpenPrice();
     
      SellTicket = OrderSend(Symbol(),OP_SELL,LotSize,Bid,0,
         0,0,"Sell Order",MagicNumber,0,Red);
      
      //Error HandLing
      if(SellTicket == -1)
      {
         ErrorCode = GetLastError();
         ErrDesc = ErrorDescription(ErrorCode);
         ErrAlert = StringConcatenate("Open Sell Order - Error ",
            ErrorCode,": ",ErrDesc);
        Alert(ErrAlert);
        
        ErrLog = StringConcatenate("Bid: ",Bid," Lots: ",LotSize);
        Print(ErrLog);
      }
      
     //Order Modification
     else
     {
      OrderSelect(SellTicket,SELECT_BY_TICKET);
      OpenPrice = OrderOpenPrice();
      
      //Calculate stop level
      StopLevel = MarketInfo(Symbol(),MODE_STOPLEVEL)*Point;
      
      RefreshRates();
      UpperStopLevel = Ask + StopLevel;
      LowerStopLevel = Bid - StopLevel;
      
      MinStop = 5 * UsePoint;
      
      //Calculate Stop Loss And TakeProfit
      if(StopLoss > 0) double SellStopLoss = OpenPrice - (StopLoss * UsePoint);
      if(TakeProfit > 0) double SellTakeProfit = OpenPrice + (TakeProfit * UsePoint);
      
      //Verify Stop Loss And TakeProfit
      if(SellStopLoss > 0 && SellStopLoss > LowerStopLevel)
      {
         SellStopLoss = LowerStopLevel - MinStop;
      }
      
      if(SellTakeProfit > 0 && BuyTakeProfit < UpperStopLevel)
         {
            SellTakeProfit = UpperStopLevel + MinStop;
         }
         
      //Modify Order
      if(IsTradeContextBusy()) Sleep(10);
      
      if(BuyStopLoss > 0 || BuyTakeProfit > 0)
         {
            TicketMod = OrderModify(BuyTicket,OpenPrice,BuyStopLoss,
               BuyTakeProfit,0);
               
          //Error Handling
          if(TicketMod == false)
          {
            ErrorCode = GetLastError();
            ErrDesc = ErrorDescription(ErrorCode);
            ErrAlert = StringConcatenate("Modify Buy Order - Error ",
               ErrorCode,": ",ErrDesc);
            Alert(ErrAlert);
            
            ErrLog = StringConcatenate("Ask: ",Ask," Bid: ",Bid," Ticket: ",
               BuyTicket," Stop: ",BuyStopLoss," Profit: ",BuyTakeProfit);
            Print(ErrLog);
          }
        }
     }
     BuyTicket = 0;
   }
   return(0);
      //Chart output for the signal
       Comment ("The Current Signal: ",signal +
            " \n The Current Risk in Percent: "+EquityPercent+
            " \n The Current Lot Size is: "+ LotSize+
            " \n Account Balance: "+ AccountBalance()+
            " \n Your Current Equity: "+ AccountEquity() );
  }
  
//+------------------------------------------------------------------+
//Pip Point Function
double PipPoint(string Currency)
   {
      int CalcDigits = MarketInfo(Currency, MODE_DIGITS);
      if(CalcDigits == 2 || CalcDigits == 4) double CalcPoint = 0.01;
      else if(CalcDigits == 3 || CalcDigits == 5) CalcPoint = 0.0001;
      return(CalcPoint);
      
   }
